from pynotice.mail.utils import *


def noticeOnFinish(sender_email, sender_authorization_code, receiver_email, mail_subject="Finish!", mail_content="", attachments=None, smtp_host=None, smtp_port=None):
    """
    When the decorated function finishes, noticeOnfinish() will send an email with the result of the function and attachments to receiver's email if the stmp of the sender's email is available.

    :param sender_email: sender's email
    :param sender_authorization_code:  sender's smtp authorization code
    :param receiver_email: receiver's email or a list of emails
    :param mail_subject: subject of the email
    :param mail_content: content of the email
    :param attachments: attachments available
    :param smtp_host: smtp host address
    :param smtp_port: smtp host port
    :return: smtp server object
    """
    assert isinstance(mail_subject, str), "mail_subject should be given a string or ''"
    assert isinstance(receiver_email, str) or isinstance(receiver_email, list), "receiver_email should be given a string or a list"

    def decorator(fun):
        def wrapper(*args, **kwargs):
            s = config_server(sender_email, sender_authorization_code, smtp_host, smtp_port)
            result = fun(*args, **kwargs)
            mail = {
                'subject': mail_subject,
                'content_text': mail_content + "\n" + str(result),
                'attachments': attachments
            }
            print("sending email...")
            s.send_mail(receiver_email, mail)
            return result
        return wrapper
    return decorator


def noticeOnException(sender_email, sender_authorization_code, receiver_email, mail_subject="Exception!", mail_content="", smtp_host=None, smtp_port=None):
    """
    When the decorated function goes wrong, noticeOnException() will send an email with the Exception Information receiver's email if the stmp of the sender's email is available.

    :param sender_email: sender's email
    :param sender_authorization_code:  sender's smtp authorization code
    :param receiver_email: receiver's email or a list of emails
    :param mail_subject: subject of the email
    :param mail_content: content of the email
    :param smtp_host: smtp host address
    :param smtp_port: smtp host port
    :return: smtp server object
    """
    assert isinstance(mail_subject, str), "mail_subject should be given a string"
    assert isinstance(mail_content, str), "mail_content should be given a string"
    assert isinstance(receiver_email, str) or isinstance(receiver_email, list), "receiver_email should be given a string or a list"

    def decorator(fun):
        def wrapper(*args, **kwargs):
            s = config_server(sender_email, sender_authorization_code, smtp_host, smtp_port)
            try:
                result = fun(*args, **kwargs)
                return result
            except Exception as e:
                mail = {
                    'subject': mail_subject,
                    'content_text': mail_content + "\n" + str(e)
                }
                print("sending email...")
                s.send_mail(receiver_email, mail)
                raise e
        return wrapper
    return decorator


from zmail import server
from typing import Optional


class SMTPConfigException(Exception):
    def __init__(self):
        err = "SMTP Server Configuration Exception\n please confirm your parameters are correct"
        Exception.__init__(self, err)


def config_server(sender_email:str, sender_autorization_code:str, smtp_host: Optional[str] = None, smtp_port: Optional[int] = None, timeout=10):
    """
    smtp server configuration

    :param sender_email: sender's email
    :param sender_autorization_code:  sender's smtp authorization code
    :param smtp_host: smtp host address
    :param smtp_port: smtp host port
    :param timeout: timeout
    :return: smtp server object
    """
    assert isinstance(sender_email, str), "sender_email should be given a string"
    assert isinstance(sender_autorization_code, str), "sender_authorization_code should be given a string"

    s = server(sender_email, sender_autorization_code, smtp_host=smtp_host, smtp_port=smtp_port, timeout=timeout)
    if s.smtp_able():
        print("server config success")
        return s
    else:
        raise SMTPConfigException






from pynotice.sound.notice import noticeOnFinish as spof, noticeOnException as spoe
from pynotice.mail.notice import noticeOnFinish as mpof, noticeOnException as mpoe
import numpy as np


# sound prompt
@spof()
def foo(name="foooo"):
    for i in range(10000):
        if i%1000==0:
            print(name)
    return "123"


# print("begin")
# ret = foo()
# print(ret)
# print("finish")
"""
out <<
begin
foooo
foooo
foooo
foooo
foooo
foooo
foooo
foooo
foooo
foooo
play audio.....
123
finish
"""

@spof()
@spoe()
def test(name="aaa"):
    print(name)
    name.append("b")
    return "ccc"

# print("begin")
# re = test()
# print(re)
# print("end")

"""
out <<
begin
aaa
play sound...
ccc
end
"""


# email prompt
sender = "1194684253@qq.com"
code = "iglraqyxfoqeghge"
receiver = "shaoeric@foxmail.com"  # list or str

@mpof(sender, code, receiver, attachments=['demo.txt'])
def test(name="aaa"):
    x = np.array([[1, 2, 3], [2, 3, 4]])
    for i in range(100):
        if i%20==0:
            print(i)
    np.savetxt("demo.txt", x)
    return x, [1,23,5,4], name

# print("begin")
# ret = test()
# print(ret)
# print("end")
"""
out <<
begin
server config success
0
20
40
60
80
sending email...
(array([[1, 2, 3],
       [2, 3, 4]]), [1, 23, 5, 4], 'aaa')
end
"""


@mpoe(sender, code, receiver)
def test(name="aaa"):
    for i in range(100):
        if i%20 == 0:
            print(i)
    x = np.array([[1,23,4]])
    np.savetxt('test.txt', x)
    y = np.loadtxt('notExist.txt')
    return y

# print("begin")
# result = test()
# print(result)
# print("end")

"""
begin
server config success
0
20
40
60
80
sending email...
Traceback (most recent call last):
...  ...
    raise IOError("%s not found." % path)
OSError: notExist.txt not found.
"""


from os.path import dirname, join, exists

sound_root = dirname(__file__)
resource_root = join(sound_root, "resource")

success_audio = join(resource_root, "success.wav")
error_audio = join(resource_root, "error.wav")



from filetype import guess
from pynotice.sound import *
import simpleaudio as sa


class FileIsNotWavFormatException(Exception):
    def __init__(self, filepath):
        self.filepath = filepath
        err = "{} is not in WAV format".format(self.filepath)
        Exception.__init__(self, err)


def check_file_type(filepath):
    filetype = guess(filepath).extension
    if filetype == "wav":
        return True
    else:
        raise FileIsNotWavFormatException(filepath)


def play(filepath=success_audio):
    wave_obj = sa.WaveObject.from_wave_file(filepath)
    play_obj = wave_obj.play()
    play_obj.wait_done()



from setuptools import setup, find_packages

with open("README.md", "r") as fh:
    long_description = fh.read()

setup(
    name='pynotice',
    version='0.0.1',
    author='shaoeric',
    author_email='shaoeric@foxmail.com',
    url='https://github.com/shaoeric/pynotice',
    description="notice by sound or email",
    long_description=long_description,
    long_description_content_type="text/markdown",
    packages=find_packages(),
    package_data={
        "": ["*.wav", "*.md", "*.txt"]
    },
    include_package_data = True,
    install_requires=['simpleaudio==1.0.2', 'zmail==0.2.5', 'filetype==1.0.5'],
    classifiers=(
        "Programming Language :: Python :: 3",
        "License :: OSI Approved :: MIT License",
        "Operating System :: OS Independent",
    )
)


#-*- coding : utf-8 -*-
# coding: utf-8

from pynotice.sound.utils import *


def noticeOnFinish(filepath=success_audio):
    """
    decorator function, when the fun finishes, noticeOnfinish() will play an audio in wav format(default is success_audio)

    :param filepath: wav audio path
    :return:
    """
    def decorator(fun):
        def wrapper(*args, **kwargs):
            check_file_type(filepath)
            result = fun(*args, **kwargs)
            play(filepath)
            return result
        return wrapper
    return decorator


def noticeOnException(filepath=error_audio):
    """
    decorator function, when fun goes wrong or raises Exception, noticeOnException() will play an audio in wav format(default is error_audio)

    :param filepath: wav audio path
    :return:
    """
    def decorator(fun):
        def wrapper(*args, **kwargs):
            check_file_type(filepath)
            try:
                result = fun(*args, **kwargs)
                return result
            except Exception as e:
                play(filepath)
                raise e
        return wrapper
    return decorator

